@@ -8,17 +8,22 @@
 #include <errno.h>
 #include <fcntl.h>
 
-#include <config.h>
-
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 
 #include <sys/socket.h>
+
+#ifdef __GLIBC__
 #include <net/ethernet.h>
+#else
+#include <linux/if_ether.h>
+#endif
+
 #include <netinet/in.h>
 
+#include <config.h>
 #ifdef HAVE_NETAX25_AX25_H
 #include <netax25/ax25.h>
 #else
@@ -30,11 +35,14 @@
 #include <netax25/kernel_rose.h>
 #endif
 
+
 #include <netax25/axlib.h>
 #include <netax25/axconfig.h>
 #include <netax25/daemon.h>
 #include <netax25/mheard.h>
 
+#include <config.h>
+
 #include "../pathnames.h"
 
 #define	KISS_MASK	0x0F
