@@ -68,7 +68,8 @@ static int readconfig(char *port)
 	int n = 0;
 	
 	if ((fp = fopen(CONF_AXPORTS_FILE, "r")) == NULL) {
-		fprintf(stderr, "%s: cannot open axports file\n", progname);
+		fprintf(stderr, "%s: cannot open axports file %s\n", 
+                        progname, CONF_AXPORTS_FILE);
 		return FALSE;
 	}
 
@@ -275,9 +276,13 @@ int main(int argc, char *argv[])
         }
 
 	if ((fd = open(argv[optind], O_RDONLY | O_NONBLOCK)) == -1) {
+            if (errno == ENOENT) {
+                fprintf(stderr, "%s: Cannot find serial device %s, no such file or directory.\n", progname, argv[optind]);
+            } else {
 		fprintf(stderr, "%s: ", progname);
 		perror("open");
-		return 1;
+            }
+	    return 1;
 	}
 
 	if (speed != 0 && !tty_speed(fd, speed))
