@@ -40,7 +40,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
-#include <sys/time.h>
+#include <time.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
